import http.client
import mimetypes
import json
from codecs import encode

class BLAPI():
    def __init__(self, blurl="sparc.biolucida.net", boundary="abc123", token="atokenstring"):
        self.__conn = None
        self.__blurl = blurl
        self.__boundary = boundary
        self.__token = token
        self.__authtoken = None
        self.__collection = None
        self.__coldict = None

    def __formblock(self, term, value):
        dataList = list()
        dataList.append(encode('Content-Disposition: form-data; name={};'.format(term)))
        dataList.append(encode('Content-Type: {}'.format('text/plain')))
        dataList.append(encode(''))
        dataList.append(encode(value))
        return dataList

    def __midboundary(self):
        return encode('--' + self.__boundary)

    def __endboundary(self):
        return encode('--' + self.__boundary + '--')
        
    def __response(self):
        return json.loads(self.__conn.getresponse().read().decode("utf-8"))

    def disconnect(self):
        if self.__conn is None:
            return False
        try:
            self.__conn.close()
            self.__conn = None
            self.__authtoken = None
            self.__collection = None
            return True
        except:
            return False

    def authenticate(self, usr, pwd):
        self.__conn = http.client.HTTPSConnection(self.__blurl)
        dataList = list()
        dataList.append(self.__midboundary())
        dataList.extend(self.__formblock("username", usr))
        dataList.append(self.__midboundary())
        dataList.extend(self.__formblock("password", pwd))
        dataList.append(self.__midboundary())
        dataList.extend(self.__formblock("token", self.__token))
        dataList.append(self.__endboundary())
        dataList.append(encode(''))
        
        payload = b'\r\n'.join(dataList)
        headers = {'Content-Type': 'multipart/form-data; boundary={}'.format(self.__boundary)}
        self.__conn.request("POST", "/api/v1/authenticate", payload, headers)

        respobj = self.__response()
        if respobj["status"] == "success":
            self.__authtoken = respobj["token"]
            #print("blapi.py: authenticated")
            return True
        else:
            #print("blapi.py: couldn't authenticate")
            return False

    def get_collections(self):
        if self.__authtoken is None:
            #print("blapi.py: not authorized")
            return None
        if self.__coldict is not None:
            #print("blapi.py: got collections")
            return self.__coldict
        headers = {"token":self.__authtoken}
        self.__conn.request("GET", "/api/v1/collections/all", "", headers)
        data = dict()
        respobj = self.__response()
        if len(respobj) > 0:
            for item in respobj:
                data[item["name"]] = item["id"]
            self.__coldict = data
            #print("blapi.py: got collections")
            return data
        else:
            #print("blapi.py: couldn't get collections")
            return None

    def choose_collection(self, colID):
        if self.__coldict is None:
            self.__coldict = self.get_collections()
        if type(colID) is str:
            if colID in self.__coldict.keys():
                colID = self.__coldict[colID]
            else:
                #print("blapi.py: couldn't choose collection")
                return False
        if self.__authtoken is None:
            #print("blapi.py: not authorized")
            return False
        headers = {"token":self.__authtoken}
        self.__conn.request("GET", "/api/v1/collections/" + str(colID), "", headers)
        data = list()
        respobj = None
        try:
            respobj = self.__response()
        except:
            #print("blapi.py: couldn't choose collection")
            return False
        if type(respobj) is list and len(respobj) > 0:
            for item in respobj:
                #print(item)
                ext = self.get_img_ext(item["id"])
                if ext not in item["image_name"]:
                    fname = item["image_name"] + "." + ext
                else:
                    fname = item["image_name"]
                data.append((fname,item["id"]))
                #print("blapi.py: " + fname)
            self.__collection = data
            #print("blapi.py: collection chosen")
            return True
        else:
            #print("blapi.py: couldn't choose collection")
            return False

    def get_img_id(self, fname):
        #print("blapi.py: "+fname)
        if self.__collection is None:
            #print("blapi.py: couldn't get image ID")
            return -2
        for img in self.__collection:
            if img[0].find(fname) > -1:
                #print("blapi.py: got image ID from filename")
                return img[1]
        #print("blapi.py: couldn't get image ID")
        return -1

    def get_img_from_id(self, imgid):
        if self.__authtoken is None:
            #print("blapi.py: not authorized")
            return "bad auth token"
        headers = {"token":self.__authtoken}
        self.__conn.request("GET", "/api/v1/image/" + str(imgid), "", headers)
        respobj = None
        try:
            respobj = self.__response()
        except:
            #print("blapi.py: couldn't get image")
            return "bad response"
        #print("blapi.py: got image from ID")
        return respobj

    def get_img_ext(self, imgid):
        if self.__authtoken is None:
            #print("blapi.py: not authorized")
            return "bad auth token"
        #print("blapi.py: getting extension from image ID")
        return self.get_img_from_id(imgid)["file_extension"]

    def get_img_xmpmeta(self, imgid):
        if self.__authtoken is None:
            #print("blapi.py: not authorized")
            return "bad auth token"
        headers = {"token":self.__authtoken}
        self.__conn.request("GET", "/api/v1/image/xmpmetadata/" + str(imgid), "", headers)
        try:
            respobj = self.__response()
        except:
            #print("blapi.py: couldn't get xmp metadata")
            return "bad response"
        #print("blapi.py: got xmp metadata from image ID")
        return respobj
