import csv, re
from enum import Enum, auto

class ManifestEditor:

    #stores columns expected in manifest
    class Columns(Enum):
        NAME = 0
        TIME = auto()
        DESC = auto()
        TYPE = auto()
        META = auto()
        BLID = auto()
        DSID = auto()
        PKID = auto()
    
    def __init__(self):
        #flag to make debug method work
        self.__debug = True
        #path to manifest CSV
        self.__mpath = str()
        #path to error log CSV
        self.__epath = str()
        #path to output CSV
        self.__wpath = str()
        #list of lists - stores entire CSV by [row][col]
        self.__data = list()
        #list of BL IDs for images, in same order as data
        self.__blids = list()
        #list of package IDs for images, in same order as data
        self.__pckids = list()
        #list of lists - each list has description and line number
        self.__errors = list()
        #regexes for compatible formats
        self.__re_comp = {
            ".jp2": re.compile(".jp2"),
            ".jpx": re.compile(".jpx"),
            ".tiff": re.compile(".tiff?"),
            ".czi": re.compile(".czi")
            }
        #regexes for vendor formats
        self.__re_vend = {
            ".lsm": re.compile(".lsm."),
            ".svs": re.compile(".svs.")
            }

    def process_manifest(self, imgformat, dsid, cmp=0):
        if len(self.__mpath) > 0:
            if self.__read_manifest(self.__mpath):
                extstr = str()
                descext = self.__check_ext()
                if descext is not None:
                    extstr = descext
                elif imgformat is not None:
                    extstr = imgformat
                else:
                    self.__mpath = str()
                    return "No image format"
                #print("editor.py:  cmp = " + str(cmp))
                self.__autofill_manifest(extstr, dsid, cmp)
                if self.__write_manifest():
                    return None
                else:
                    self.__mpath = str()
                    return "Couldn't write file"
            else:
                self.__mpath = str()
                return "Couldn't read file"
        self.__mpath = str()
        return "Invalid path"

    def check_manifest(self, fpath):
        if self.__read_manifest(fpath):
            if self.__check_ext() is not None:
                return None
            else:
                return 1
        else:
            return 2

    def validate_ext(self, txt):
        for ext in self.__re_vend.keys():
            if self.__re_vend[ext].search(txt) is not None:
                return True
        return False

    def get_vend_exts(self):
        return list(self.__re_vend.keys())

    def get_filenames(self):
        fnamelist = list()
        for row in self.__data:
            fnamelist.append(row[self.Columns.NAME.value])
        return fnamelist[1:]

    def set_blids(self, blidlist):
        if len(blidlist) == len(self.__data) - 1:
            self.__blids = blidlist
            return True
        return False

    def set_pckids(self, pckidlist):
        if len(pckidlist) == len(self.__data) - 1:
            self.__pckids = pckidlist
            return True
        return False
        
    def __read_manifest(self, fpath):
        self.__mpath = fpath
        self.__epath = fpath.rsplit(".",1)[0] + "_errors.csv"
        self.__wpath = fpath.rsplit(".",1)[0] + "_curated.csv"
        self.__data.clear()
        self.__errors.clear()
        #read entire CSV into self.__data and add BLID column
        try:
            with open(fpath, mode="r", newline="\r\n") as csvfile:
                reader = csv.reader(csvfile)
                for row in reader:
                    if len(row) < len(self.Columns):
                        row.extend([""]*(len(self.Columns)-len(row)))
                    self.__data.append(row)
            self.__data[0][self.Columns.BLID.value] = "Biolucida ID"
            self.__data[0][self.Columns.DSID.value] = "Pennsieve Dataset ID"
            self.__data[0][self.Columns.PKID.value] = "Pennsieve Package ID"
            return True
        except:
            return False

    def __check_ext(self):
        #scan filename column for known vendor formats
        descext = None
        for rowind in range(1, len(self.__data)):
            descext = self.__get_desc_ext(self.__data[rowind][self.Columns.NAME.value])
            if descext is not None:
                break
        if descext is None:
            return None
        return descext

    def __autofill_manifest(self, descext, dsid, cmp=0):
        #print("editor.py:   ext = " + descext)
        for rowind in range(1, len(self.__data)):
            #print("  editor.py: pass " + str(rowind) + " in autofill")
            #truncate names
            #print("  editor.py:  truncating")
            #slashind = self.__data[rowind][self.Columns.NAME.value].rfind("/")
            #if slashind > -1:
            #    self.__data[rowind][self.Columns.NAME.value] = self.__data[rowind][self.Columns.NAME.value][slashind+1:]
            #meta
            #print("  editor.py:  metadata")
            metaext = self.__get_meta_ext(self.__data[rowind][self.Columns.TYPE.value])
            if metaext is not None:
                self.__data[rowind][self.Columns.META.value] = self.__make_meta_str(metaext)
            elif self.__debug:
                self.__errors.append(self.__make_err(rowind + 1, "METADATA", "is not jp2, jpx, or tiff"))
            #print("  editor.py:  metaext type: " + str(type(metaext)))
            #print("  editor.py:  metaext: " + str(metaext))
            #desc
            #print("  editor.py:  description")
            if metaext is not None:
                self.__data[rowind][self.Columns.DESC.value] = self.__make_desc_str(self.__data[rowind][self.Columns.NAME.value], descext, metaext, cmp)
            #blids
            #print("  editor.py:  add BL ID")
            self.__data[rowind][self.Columns.BLID.value] = self.__blids[rowind-1]
            #dsid
            self.__data[rowind][self.Columns.DSID.value] = dsid
            #pkid
            self.__data[rowind][self.Columns.PKID.value] = self.__pckids[rowind-1]

    def __write_manifest(self):
        try:
            with open(self.__wpath, mode="w", newline="") as csvfile:
                writer = csv.writer(csvfile)
                for row in self.__data:
                    writer.writerow(row)
            if self.__debug:
                if len(self.__errors) > 0:
                    with open(self.__epath, mode="w", newline="") as errfile:
                        errwriter = csv.writer(errfile)
                        errwriter.writerow(["Manifest Row", "Manifest Column", "Error Description"])
                        for err in self.__errors:
                            errwriter.writerow(err)
            return True
        except:
            return False

    def debug(self):
        if self.__debug:
            print("self.__mpath:")
            print(self.__mpath)
            print()
            print("self.__epath:")
            print(self.__epath)
            print()
            print("self.__data:")
            print(self.__data)
            print()
            print("self.__errors:")
            print(self.__errors)
            print()

    def __get_meta_ext(self, ftype):
        for ext in self.__re_comp.keys():
            if self.__re_comp[ext].search(ftype) is not None:
                return ext
        return None

    def __make_meta_str(self, metaext):
        return "image/vnd.ome.xml+{metaext}".format(metaext=metaext[1:])

    def __get_desc_ext(self, fname):
        for ext in self.__re_vend.keys():
            if self.__re_vend[ext].search(fname) is not None:
                return ext
        return None

    def __make_desc_str(self, fname, fext, aftertype, cmp=0):
        beforestr = "primary/" + fname.rsplit(".", 1)[0] + fext
        #print("    editor.py: " + beforestr)
        #print("    editor.py: " + type(aftertype))
        #print("    editor.py: " + aftertype)
        if self.__re_comp[".tiff"].search(aftertype) is not None:
            aftertype = "OME.TIFF"
        #print("    editor.py: " + aftertype)
        cmpstr = " "
        if cmp > 0 and aftertype != "OME.TIFF":
            cmpstr = " with {cmp}:1 compression ".format(cmp = str(cmp))
        #print("    editor.py: " + cmpstr)
        boilerplate = "Image derived from {beforestr}. Converted to {aftertype}{cmpstr}using MicroFile+ (RRID:SCR_018724) from MBF Bioscience. Microscopy metadata included in the file header.".format(beforestr=beforestr, aftertype=aftertype, cmpstr=cmpstr)
        #print("    editor.py: " + boilerplate)
        return boilerplate

    def __make_err(self, row, errhead, errtext):
        return [row, errhead, errtext]
